import { Injectable } from '@angular/core';
import { Themes } from 'src/app/models/view/themes.enum';
import { AuthService } from '../auth/auth.service';

@Injectable({
  providedIn: 'root'
})
export class ThemeService {

  themeName: string | null;

  sidebarWidth = '150px';
  isSidebarOpen = false;
  hasLangChanged = false;

  constructor(
    private authService: AuthService
  ) {
    this.loadTheme();
    this.loadMenu();
    this.themeName = null;
  }

  loadTheme(): void {
    const token = this.authService.getDecodedToken();
    const mail = this.authService.getEMailFromDecodedToken(token);

    let defaultThemeName = localStorage.getItem(`default_themeName`);
    if (!defaultThemeName || defaultThemeName != Themes.Default) {
      defaultThemeName = Themes.Default;
    }

    if (!mail) {
      this.setTheme(defaultThemeName);
      return;
    }

    let userThemeName = localStorage.getItem(`${mail}_themeName`);
    if (!userThemeName) {
      this.setTheme(defaultThemeName);
      return;
    }
    this.setTheme(userThemeName);
  }

  setTheme(name: string): void {
    this.authService.isUserLoggedInAsync().then(result => {
      if (!result) {
        localStorage.setItem(`default_themeName`, Themes.Default);
        this.themeName = Themes.Default;
      }

      const token = this.authService.getDecodedToken();
      const mail = this.authService.getEMailFromDecodedToken(token);
      if (mail) {
        localStorage.setItem(`${mail}_themeName`, name);
      }
      localStorage.setItem(`default_themeName`, name);
      this.themeName = name;
    });
  }

  loadMenu() {
    const token = this.authService.getDecodedToken();
    const mail = this.authService.getEMailFromDecodedToken(token);

    if (!mail) {
      this.setSideWidth(true);
      return;
    }

    let isMenuOpen = true;
    let isMenuOpenStr = localStorage.getItem(`${mail}_isMenuOpen`);
    if (isMenuOpenStr) {
      isMenuOpen = Boolean(isMenuOpenStr);
    }

    this.setIsMenuOpen(isMenuOpen);
  }

  setIsMenuOpen(isMenuOpen: boolean) {
    const token = this.authService.getDecodedToken();
    const mail = this.authService.getEMailFromDecodedToken(token);

    if (!mail) {
      this.setSideWidth(true);
      return;
    }

    localStorage.setItem(`${mail}_isMenuOpen`, isMenuOpen + "");
    this.setSideWidth(isMenuOpen);
  }

  setSideWidth($event: any): void {
    this.sidebarWidth = $event ? '150px' : '50px';
    this.isSidebarOpen = $event;
  }
}
